#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const glob = require("glob");
const nopt = require("nopt");
const logger = require("consola");
const path = require("path");
const fs = require("fs");
const promised_1 = require("@znck/promised");
const plugins_1 = require("../plugins");
const index_1 = require("../index");
const utils_1 = require("../utils");
function run(source, dest, files, { plugins = [], toStdOut = false, overwrite = false, silent = false }) {
    return __awaiter(this, void 0, void 0, function* () {
        let hasAnyErrors = false;
        const duplicates = new Map();
        yield utils_1.all(files
            // Compile.
            .map((from) => __awaiter(this, void 0, void 0, function* () {
            try {
                const { filename, errors, tips, code } = yield index_1.compile(path.resolve(source, from), yield utils_1.read(path.resolve(source, from)), [...plugins_1.default, ...plugins]);
                const hasErrors = errors.length > 0;
                if (hasErrors) {
                    hasAnyErrors = true;
                    logger.error('in ' + from);
                    errors.forEach((error) => console.error(error));
                }
                if (tips.length && !silent) {
                    logger.info('for ' +
                        from +
                        '\n' +
                        tips.map((tip) => '  - ' + tip.replace(/\n/g, '\n  ')).join('\n'));
                }
                if (!hasErrors && filename && code) {
                    const to = path.relative(source, filename);
                    if (toStdOut) {
                        console.log(code);
                    }
                    else if (!duplicates.has(to)) {
                        try {
                            yield utils_1.write(path.resolve(dest, to), code, overwrite);
                            !silent && logger.log('> ' + from + ' -> ' + to);
                        }
                        catch (e) {
                            logger.error(e.message);
                        }
                        duplicates.set(to, from);
                    }
                    else {
                        logger.fatal(`Both '${duplicates.get(to)}' and ${from} are compiled to ${to}.`);
                        logger.info('Rename one of the above files.');
                    }
                }
            }
            catch (e) {
                logger.fatal('while processing ' + from, e.stack);
            }
        })));
        return hasAnyErrors;
    });
}
function main(argv) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = nopt({ outDir: String, force: Boolean, silent: Boolean }, { f: '--force', d: '--outDir', s: '--silent' }, argv, 2);
        let config = {
            force: options.force,
            paths: options.argv.remain.length ? options.argv.remain : ['src'],
            plugins: [],
            silent: options.silent,
            target: options.outDir
                ? path.resolve(process.cwd(), options.outDir)
                : path.join(process.cwd(), 'dist')
        };
        const configPath = path.join(process.cwd(), 'vuec.config.js');
        const hasConfigFile = yield promised_1.default(fs).exists(configPath);
        if (hasConfigFile) {
            logger.info('Using config from vuec.config.js.');
            const local = require(configPath);
            if (typeof local === 'function') {
                config = Object.assign({}, config, local(config));
            }
            else {
                config = Object.assign({}, config, local);
            }
        }
        let hasAnyErrors = true;
        for (const filename of config.paths) {
            const file = path.resolve(process.cwd(), filename);
            if (!(yield promised_1.default(fs).exists(file))) {
                logger.error(`No such file or directory, ${filename}`);
                continue;
            }
            const isFile = (yield promised_1.default(fs).lstat(file)).isFile();
            const dir = isFile ? path.dirname(file) : file;
            const dest = config.paths.length > 1 && !isFile ? path.join(config.target, filename) : config.target;
            const files = isFile
                ? [file]
                : yield promised_1.default({ glob }).glob('**', { cwd: dir, nodir: true });
            hasAnyErrors = yield run(dir, dest, files, {
                plugins: config.plugins,
                toStdOut: !hasConfigFile && !options.outDir && config.paths.length === 1 && isFile,
                silent: config.silent,
                overwrite: config.force
            });
        }
        if (hasAnyErrors === true)
            process.exit(1);
    });
}
if (process.argv.find(arg => arg === '-h' || arg === '--help')) {
    console.log(`
  Usage: vuec [options] [file ...]

  Examples: vuec
            vuec --outDir dist app.vue

  Options:
  -h, --help              Output usage information.
  -d <path>
  --outDir <path>         Redirect output structure to the directory.
  -f, --force             Overwrite existing files.
  -s, --silent            No console output.
  `);
}
else {
    main(process.argv).catch((error) => logger.fatal(error));
}
