"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const component_compiler_utils_1 = require("@vue/component-compiler-utils");
class MessageBag {
    constructor() {
        this.scope = '';
        this.errors = [];
        this.tips = [];
    }
    setScope(name) {
        this.scope = name;
    }
    error(error) {
        this.errors.push(error);
    }
    tip(tip) {
        this.tips.push(tip);
    }
    hasError() {
        return this.errors.length > 0;
    }
    write() {
        return { tips: this.tips, errors: this.errors };
    }
}
exports.MessageBag = MessageBag;
class FileRewriter extends MessageBag {
    constructor(input) {
        super();
        this.output = {};
        this.input = input;
    }
    name(filename) {
        this.output.filename = filename;
    }
    content(content) {
        this.output.content = content;
    }
    write() {
        if (this.hasError())
            return super.write();
        return Object.assign({}, super.write(), { filename: this.output.filename || this.input.filename, code: this.output.content });
    }
}
exports.FileRewriter = FileRewriter;
class ComponentRewriter extends MessageBag {
    constructor(filename, descriptor) {
        super();
        this.mixins = [];
        this.properties = {};
        this.hooks = {};
        this.blocks = [];
        this.filename = filename;
        this.descriptor = descriptor;
        this.isFunctional = !!(descriptor.template && 'functional' in descriptor.template.attrs);
    }
    forEach(callback) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.descriptor.template)
                yield callback(normalizeBlock(this.descriptor.template, this.filename));
            if (this.descriptor.script)
                yield callback(normalizeBlock(this.descriptor.script, this.filename));
            for (const style of this.descriptor.styles) {
                yield callback(normalizeBlock(style, this.filename));
            }
            for (const block of this.descriptor.customBlocks) {
                yield callback(normalizeBlock(block, this.filename));
            }
        });
    }
    defineInstanceProperty(name, value) {
        if (name in this.properties || ComponentRewriter.CONSTANTS.properties.has(name)) {
            this.tip('Plugin (' +
                this.currentPluginName +
                ') is trying to overwrite existing instance property.');
        }
        else {
            this.properties[name] = value;
        }
    }
    defineOption(name, value) {
        this.mixins[this.mixins.length - 1][name] = value;
    }
    defineHook(name, value) {
        if (this.isFunctional) {
            // TODO: Handle hooks!.
        }
        else if (name in this.hooks) {
            this.hooks[name].push(value);
        }
        else {
            this.hooks[name] = [value];
        }
    }
    block(block) {
        if (block.type === 'template' || block.type === 'script') {
            if (this.blocks.some(it => it.type === block.type)) {
                throw new Error(`Only one ${block.type} block is supported.`);
            }
        }
        this.blocks.push(block);
    }
    createBlock(type, attrs, code) {
        this.block({ filename: '', type, attrs, code, content: '' });
    }
    addBlock(block, attrs, content) {
        if (typeof block === 'string')
            this.createBlock(block, attrs || {}, content);
        else if (block && typeof block === 'object')
            this.block(block);
    }
    write() {
        if (this.hasError())
            return super.write();
        const code = this.blocks.map(stringifyBlock).join('\n');
        return Object.assign({}, super.write(), { filename: this.filename, code });
    }
}
ComponentRewriter.CONSTANTS = {
    properties: new Set(['$on'])
};
exports.ComponentRewriter = ComponentRewriter;
// -- Internal Utility Functions --
function stringifyBlock(block) {
    const { type, code, attrs } = block;
    const attrString = Object.keys(attrs)
        .map(key => (attrs[key] === '' || attrs[key] === true ? key : key + '=' + JSON.stringify(attrs[key])))
        .join(' ');
    return `<${type}${attrString ? ' ' + attrString : ''}>\n${(code || '').trim()}\n</${type}>\n`;
}
function normalizeBlock(block, filename) {
    delete block.attrs.lang;
    const output = {
        filename,
        type: block.type,
        attrs: block.attrs,
        content: block.content,
        start: block.start,
        end: block.end,
        map: block.map,
        lang: block.lang
    };
    const keys = ['module', 'scope', 'src'];
    keys.forEach((key) => {
        if (key in block) {
            output.attrs[key] = typeof block[key] !== 'string'
                ? ''
                : block[key];
        }
    });
    return output;
}
function compile(filename, content, plugins) {
    return __awaiter(this, void 0, void 0, function* () {
        let isVue = /\.vue$/gi.test(filename);
        if (!isVue) {
            const file = { filename, content };
            const api = new FileRewriter(file);
            for (const plugin of plugins) {
                if (plugin.testFile && plugin.testFile(filename)) {
                    yield plugin.processFile(file, api);
                }
            }
            return api.write();
        }
        else {
            const api = new ComponentRewriter(filename, component_compiler_utils_1.parse({ filename: filename, source: content }));
            yield api.forEach((block) => __awaiter(this, void 0, void 0, function* () {
                for (const plugin of plugins) {
                    if (plugin.testBlock && plugin.testBlock(block.type, block.attrs.src || block.lang)) {
                        yield plugin.processBlock(Object.assign({}, block, { attrs: Object.assign({}, block.attrs) }), api);
                    }
                }
            }));
            return api.write();
        }
    });
}
exports.compile = compile;
